@@ -36,10 +36,10 @@ from django_filters.filters import (
     UUIDFilter,
 )
 
+from django_property_filter.constants import EMPTY_VALUES
 from django_property_filter.utils import (
     compare_by_lookup_expression,
     get_value_for_db_field,
-    sort_queryset
 )
 
 logger = logging.getLogger(__name__)
@@ -82,14 +82,14 @@ class PropertyBaseFilter(Filter):
         # Filtering is done via filter_pks() via PropertyFilterset, raise Exception if wrongly configured
         raise ImproperlyConfigured('Invalid call to filter(), make sure to use PropertyFilterSet instead of Filterset')
 
-    def filter_pks(self, initial_pk_list, queryset, value):
+    def filter_pks(self, initial_pk_list, queryset, value, *, or_pk_list=None):
         """
         Filter the Given Queryset against the given value and return a list of matching Primary Keys.
 
         if initial_pk_list is not None only those Primary Keys will be considered
         """
         # If no Value given we don't need to filter at all
-        if not value and value != 0:
+        if value in EMPTY_VALUES:
             return initial_pk_list
 
         # Not None but empty List, Nothing to do, No chance for a find
@@ -99,15 +99,20 @@ class PropertyBaseFilter(Filter):
         # Filter all values from queryset, get the pk list
         wanted_pks = set()
         for obj in queryset:
+            # If we have an initial pk list (AND condition) we must check only those items
+            if initial_pk_list is not None and obj.pk not in initial_pk_list:
+                continue
+
+            # If we have an or_pk_list then those entries we don't need to check anymore and can just add them
+            if or_pk_list is not None and obj.pk in or_pk_list:
+                wanted_pks.add(obj.pk)
+                continue
+
             property_value = get_value_for_db_field(obj, self.property_fld_name)
             if self._compare_lookup_with_qs_entry(self.lookup_expr, value, property_value):
                 wanted_pks.add(obj.pk)
 
-        # Find Entries in both lists if original provided
-        if initial_pk_list is not None:  # We have initial pk list, only return joined results
-            wanted_pks = wanted_pks & set(initial_pk_list)
-
-        return list(wanted_pks)
+        return wanted_pks
 
     def verify_lookup(self, lookup_expr):
         """Check if lookup_expr is supported."""
@@ -142,9 +147,10 @@ class ChoiceConvertionMixin():  # pylint: disable=too-few-public-methods
         new_lookup_value = lookup_value
         new_property_value = property_value
 
-        if type(lookup_value) != type(property_value):  # pylint: disable=unidiomatic-typecheck
+        property_type = type(property_value)
+        if type(lookup_value) != property_type:  # pylint: disable=unidiomatic-typecheck
             try:
-                convert_lookup_value = type(property_value)(lookup_value)
+                convert_lookup_value = property_type(lookup_value)
             except (ValueError, TypeError):
                 pass
             else:
@@ -174,9 +180,10 @@ class PropertyBaseCSVFilter(PropertyBaseFilter, BaseCSVFilter):
             if not entry and not isinstance(property_value, str):
                 raise ValueError(F'Empty value not allowed for type "{type(property_value)}"')
 
-            if type(entry) != type(property_value):  # pylint: disable=unidiomatic-typecheck
+            property_type = type(property_value)
+            if type(entry) != property_type:  # pylint: disable=unidiomatic-typecheck
                 try:
-                    convert_lookup_value = type(property_value)(entry)
+                    convert_lookup_value = property_type(entry)
                 except (ValueError, TypeError):
                     # Use original if can't convert
                     pass
@@ -240,41 +247,38 @@ class PropertyLookupChoiceFilter(ChoiceConvertionMixin, PropertyBaseFilter, Look
 
         return lookup_tup_list
 
-    def filter_pks(self, initial_pk_list, queryset, value):
+    def filter_pks(self, initial_pk_list, queryset, value, **kwargs):
         """Perform the custom filtering."""
-        if not value:
-            return super().filter_pks(initial_pk_list, queryset, None)
-
         self.lookup_expr = value.lookup_expr
-        return super().filter_pks(initial_pk_list, queryset, value.value)
+        return super().filter_pks(initial_pk_list, queryset, value.value, **kwargs)
 
 
 class PropertyMultipleChoiceFilter(ChoiceConvertionMixin, PropertyBaseFilter, MultipleChoiceFilter):
     """Adding Property Support to MultipleChoiceFilter."""
 
-    def filter_pks(self, initial_pk_list, queryset, value):
+    def filter_pks(self, initial_pk_list, queryset, value, **kwargs):  # pylint: disable=unused-argument
         """Filter Multiple Choice Property Values."""
-        # If no Value given we don't need to filter at all
-        if not value:
-            return initial_pk_list
-
         # Not None but empty List, Nothing to do, No chance for a find
         if not queryset:
             return []
 
-        result_pks = None
-        for sub_value in value:
-            filter_result = set(super().filter_pks(None, queryset, sub_value))
-            if self.conjoined:  # AND
-                if result_pks is None:
-                    result_pks = set(initial_pk_list)
-                result_pks &= filter_result
-            else:  # OR
-                if result_pks is None:
-                    result_pks = set()
-                result_pks |= filter_result
+        result_pks = and_list = or_list = None
 
-        return list(result_pks) if result_pks is not None else []
+        if self.conjoined:  # AND
+            and_list = set(initial_pk_list)
+        else:  # OR
+            or_list = None
+
+        # Passing in and_list ad or_list to skipe duplicate tests and avoid &= and |= of sets here
+        for sub_value in value:
+            filter_result = set(super().filter_pks(and_list, queryset, sub_value, or_pk_list=or_list))
+
+            if self.conjoined:  # AND
+                result_pks = and_list = filter_result
+            else:  # OR
+                result_pks = or_list = filter_result
+
+        return result_pks if result_pks is not None else set()
 
 
 class PropertyNumberFilter(PropertyBaseFilter, NumberFilter):
@@ -341,10 +345,10 @@ class PropertyAllValuesFilter(PropertyChoiceFilter, AllValuesFilter):
         """Filed Property to setup default choices."""
         queryset = self.model._default_manager.distinct()  # pylint: disable=no-member,protected-access
 
-        value_list = []
+        value_list = set()
         for obj in queryset:
             property_value = get_value_for_db_field(obj, self.property_fld_name)
-            value_list.append(property_value)
+            value_list.add(property_value)
 
         value_list = sorted(value_list, key=lambda x: (x is None, x))
 
@@ -362,12 +366,12 @@ class PropertyAllValuesMultipleFilter(PropertyMultipleChoiceFilter, AllValuesMul
         """Filed Property to setup default choices."""
         queryset = self.model._default_manager.distinct()  # pylint: disable=no-member,protected-access
 
-        value_list = []
+        value_list = set()
         for obj in queryset:
             property_value = get_value_for_db_field(obj, self.property_fld_name)
-            value_list.append(property_value)
+            value_list.add(property_value)
 
-        value_list = sorted(set(value_list), key=lambda x: (x is None, x))
+        value_list = sorted(value_list, key=lambda x: (x is None, x))
 
         self.extra['choices'] = [(prop, str(prop)) for prop in value_list]
 
@@ -459,16 +463,38 @@ class PropertyOrderingFilter(  # pylint: disable=too-many-ancestors
 
     supported_lookups = ['exact']
 
-    def filter_pks(self, initial_pk_list, queryset, value):
+    def __init__(self, *args, **kwargs):
+        """Constructr for PropertyOrderingFilter."""
+        kwargs.setdefault('label', 'Property Ordering')
+        super().__init__(*args, **kwargs)
+
+    def filter_pks(self, initial_pk_list, queryset, value, **kwargs):  # pylint: disable=unused-argument
         """Filter the PropertyOrderingFilter."""
-        # If no value is set just return this queryset
-        if not value:
-            return initial_pk_list
-
         # Only sort by the first parameter
-        sorted_qs = sort_queryset(self.get_ordering_value(value[0]), queryset)
+        return self.sorted_pk_list_from_property(self.get_ordering_value(value[0]), queryset)
 
-        return list(sorted_qs.values_list('pk', flat=True))
+    def sorted_pk_list_from_property(self, sort_property, queryset):  # pylint: disable=no-self-use
+        """Sorting the primary key list of the given queryset based on the given property."""
+        # Identify the sort order
+        descending = False
+        if sort_property.startswith('-'):
+            descending = True
+            sort_property = sort_property[1:]
+
+        # Build a list of pk and value, this might become very large depending on data type
+        # Need to use a list because set will loose order
+        value_list = []
+        for obj in queryset:
+            property_value = get_value_for_db_field(obj, sort_property)
+            value_list.append((obj.pk, property_value))
+
+        # Sort the list of tuples
+        value_list = sorted(value_list, key=lambda x: x[1], reverse=descending)
+
+        # Get a list of sorted primary keys
+        value_list = [entry[0] for entry in value_list]
+
+        return value_list
 
 
 class PropertyTimeRangeFilter(PropertyRangeFilter, TimeRangeFilter):
@@ -487,3 +513,7 @@ EXPLICIT_ONLY_FILTERS = [
     PropertyTypedChoiceFilter,
     PropertyTypedMultipleChoiceFilter,
 ]
+
+PRESERVE_ORDER_FILTERS = [
+    PropertyOrderingFilter,
+]
