@@ -24,7 +24,7 @@ def get_db_version():
     return 'Unknown'
 
 
-def convert_int_list_to_range_lists(int_list):
+def convert_int_list_to_range_lists(int_list, *, sort_list=True):
     """
     Convert a list of numbers to ranges and returns a list of tuples representing the ranges.
 
@@ -32,7 +32,11 @@ def convert_int_list_to_range_lists(int_list):
     """
     # Build a list of lists
     range_list = []
-    for num in sorted(int_list):
+    working_list = int_list
+    if sort_list:
+        working_list = sorted(int_list)
+
+    for num in working_list:
         if range_list:
             # Check if Part of range
             if range_list[-1][1] + 1 == num:  # Continuing a range
@@ -88,12 +92,13 @@ def build_limited_filter_expr(pk_list, max_params):
 
     # Just go until we used up max_params parameters
     in_range_list = []      # Each entry takes up 2 parameters
-    in_list = []            # Each entry takes up 1 parameter
+    in_list = set()         # Each entry takes up 1 parameter
 
     params_used = 0
+
     for entry in sort_range_list(convert_int_list_to_range_lists(pk_list), descending=True):
         if entry[0] == entry[1] or params_used + 1 >= max_params:  # single item or space for only 1 param
-            in_list.append(entry[0])
+            in_list.add(entry[0])
             params_used += 1
         else:  # Range item and enough space for whole range
             in_range_list.append(Q(pk__range=[entry[0], entry[1]]))
@@ -111,7 +116,7 @@ def build_limited_filter_expr(pk_list, max_params):
     return range_filter_expr
 
 
-def filter_qs_by_pk_list(queryset, pk_list):
+def filter_qs_by_pk_list(queryset, pk_list, *, preserve_order=None):
     """Filter the given queryset by the given list of primary keys.
 
     Our current approach to use "pk__in" has a big drawback in sqlite where by default
@@ -121,58 +126,49 @@ def filter_qs_by_pk_list(queryset, pk_list):
     https://www.sqlite.org/limits.html#:~:text=To%20prevent%20excessive%20memory%20allocations,0.
     9. Maximum Number Of Host Parameters In A Single SQL Statement
     """
-    preserved = Case(*[When(pk=pk, then=pos) for pos, pk in enumerate(pk_list)])
-    result_qs = queryset.filter(pk__in=pk_list).order_by(preserved)
+    result_qs = queryset.filter(pk__in=pk_list)
 
     # Only evaluate if we know how to limit the list
     # e.g. For sqlite we know the default limits per version, if we exceed those we can limit how much we return.
     # For other DBs we currently don't know so if there is a limit we just let the exception be passed on
 
     max_params = get_max_params_for_db()
-    # No need to try again if we don't know the safe max or we have less items than the safe max in the first place
     if max_params is not None and max_params < len(pk_list):
         try:
             # Evaluate the Result
             result_qs.count()
         except OperationalError:
-            max_params = get_max_params_for_db()
-            if max_params is not None and max_params < len(pk_list):
+            if preserve_order:
+                # Only do 1/3 of the items to be able to preserve the order
+                items_left = int(max_params / 3)
+                limited_pk_list = set()
+
+                for entry in preserve_order:
+                    if items_left <= 0:
+                        break
+
+                    if entry in pk_list:
+                        limited_pk_list.add(entry)
+                        items_left -= 1
+                preserve_order = limited_pk_list  # Order preserved for limited pks
+                result_qs = queryset.filter(pk__in=limited_pk_list)
+
+                logging.warning('Limiting the Max SQL Parameters to be able to preserve the filter order')
+
+            else:
                 range_filter_expr = build_limited_filter_expr(pk_list, max_params)
                 result_qs = queryset.filter(range_filter_expr)
 
-                logging.warning(F'Only returning the first {result_qs.count()} items because of max parameter'
-                                F'limitations of Database "{get_db_vendor()}" with version "{get_db_version()}"')
+            logging.warning(F'Only returning the first {result_qs.count()} items because of max parameter '
+                            F'limitations of Database "{get_db_vendor()}" with version "{get_db_version()}"')
+
+    if preserve_order:
+        preserved = Case(*[When(pk=pk, then=pos) for pos, pk in enumerate(preserve_order)])
+        result_qs = result_qs.order_by(preserved)
 
     return result_qs
 
 
-def sort_queryset(sort_property, queryset):
-    """Sort the queryset by the given property name. "-" for descending is supported."""
-    # Identify the sort order
-    descending = False
-    if sort_property.startswith('-'):
-        descending = True
-        sort_property = sort_property[1:]
-
-    # Build a list of pk and value, this might become very large depending on data type
-    value_list = []
-    for obj in queryset:
-        property_value = get_value_for_db_field(obj, sort_property)
-        value_list.append((obj.pk, property_value))
-
-    # Sort the list of tuples
-    value_list = sorted(value_list, key=lambda x: x[1], reverse=descending)
-
-    # Get a list of sorted primary keys
-    value_list = [entry[0] for entry in value_list]
-
-    # Sort the Queryset
-    preserved = Case(*[When(pk=pk, then=pos) for pos, pk in enumerate(value_list)])
-    queryset = filter_qs_by_pk_list(queryset, value_list).order_by(preserved)
-
-    return queryset
-
-
 def get_value_for_db_field(obj, field_str):
     """Lookup a model field or property."""
     def get_attr_val_recursive(obj, sub_list):
